@@ -21,13 +21,16 @@ import threading
 import time
 from typing import IO, Any, Optional, TextIO
 
-__version__ = "0.8.0"
+__version__ = "0.10.1"
 __schema_version__ = "0.2.0"
 
 
 lgr = logging.getLogger("con-duct")
 DEFAULT_LOG_LEVEL = os.environ.get("DUCT_LOG_LEVEL", "INFO").upper()
 
+DUCT_OUTPUT_PREFIX = os.getenv(
+    "DUCT_OUTPUT_PREFIX", ".duct/logs/{datetime_filesafe}-{pid}_"
+)
 ENV_PREFIXES = ("PBS_", "SLURM_", "OSG")
 SUFFIXES = {
     "stdout": "stdout",
@@ -45,10 +48,10 @@ EXECUTION_SUMMARY_FORMAT = (
     "Memory Average Usage (RSS): {average_rss!S}\n"
     "Virtual Memory Peak Usage (VSZ): {peak_vsz!S}\n"
     "Virtual Memory Average Usage (VSZ): {average_vsz!S}\n"
-    "Memory Peak Percentage: {peak_pmem!N}%\n"
-    "Memory Average Percentage: {average_pmem!N}%\n"
-    "CPU Peak Usage: {peak_pcpu!N}%\n"
-    "Average CPU Usage: {average_pcpu!N}%\n"
+    "Memory Peak Percentage: {peak_pmem:.2f!N}%\n"
+    "Memory Average Percentage: {average_pmem:.2f!N}%\n"
+    "CPU Peak Usage: {peak_pcpu:.2f!N}%\n"
+    "Average CPU Usage: {average_pcpu:.2f!N}%\n"
 )
 
 
@@ -648,13 +651,21 @@ class SummaryFormatter(string.Formatter):
         if value is None:
             # TODO: could still use our formatter and make it red or smth like that
             return self.NONE
+        # if it is a composite :format!conversion, we need to split it
+        if "!" in format_spec and format_spec.index("!") > 1:
+            format_spec, conversion = format_spec.split("!", 1)
+        else:
+            conversion = None
         try:
-            return super().format_field(value, format_spec)
-        except ValueError:
+            value_ = super().format_field(value, format_spec)
+        except ValueError as exc:
             lgr.warning(
-                f"Value: {value} is invalid for format spec {format_spec}, falling back to `str`"
+                f"Falling back to `str` formatting for {value!r} due to exception: {exc}"
             )
             return str(value)
+        if conversion:
+            return self.convert_field(value_, conversion)
+        return value_
 
 
 @dataclass
@@ -664,6 +675,7 @@ class Arguments:
     output_prefix: str
     sample_interval: float
     report_interval: float
+    fail_time: float
     clobber: bool
     capture_outputs: Outputs
     outputs: Outputs
@@ -704,9 +716,7 @@ class Arguments:
             "-p",
             "--output-prefix",
             type=str,
-            default=os.getenv(
-                "DUCT_OUTPUT_PREFIX", ".duct/logs/{datetime_filesafe}-{pid}_"
-            ),
+            default=DUCT_OUTPUT_PREFIX,
             help="File string format to be used as a prefix for the files -- the captured "
             "stdout and stderr and the resource usage logs. The understood variables are "
             "{datetime}, {datetime_filesafe}, and {pid}. "
@@ -764,6 +774,16 @@ class Arguments:
             default=float(os.getenv("DUCT_REPORT_INTERVAL", "60.0")),
             help="Interval in seconds at which to report aggregated data.",
         )
+        parser.add_argument(
+            "--fail-time",
+            "--f-t",
+            type=float,
+            default=float(os.getenv("DUCT_FAIL_TIME", "3.0")),
+            help="If command fails in less than this specified time (seconds), duct would remove logs. "
+            "Set to 0 if you would like to keep logs for a failing command regardless of its run time. "
+            "Set to negative (e.g. -1) if you would like to not keep logs for any failing command.",
+        )
+
         parser.add_argument(
             "-c",
             "--capture-outputs",
@@ -799,6 +819,7 @@ class Arguments:
             output_prefix=args.output_prefix,
             sample_interval=args.sample_interval,
             report_interval=args.report_interval,
+            fail_time=args.fail_time,
             capture_outputs=args.capture_outputs,
             outputs=args.outputs,
             record_types=args.record_types,
@@ -946,6 +967,14 @@ def safe_close_files(file_list: Iterable[Any]) -> None:
             pass
 
 
+def remove_files(log_paths: LogPaths, assert_empty: bool = False) -> None:
+    for _, file_path in log_paths:
+        if os.path.exists(file_path):
+            if assert_empty:
+                assert os.stat(file_path).st_size == 0
+            os.remove(file_path)
+
+
 def main() -> None:
     logging.basicConfig(
         format="%(asctime)s [%(levelname)-8s] %(name)s: %(message)s",
@@ -1005,10 +1034,7 @@ def execute(args: Arguments) -> int:
         # We should remove log etc files since they are 0-sized
         # degenerates etc
         safe_close_files(files_to_close)
-        for _, file_path in log_paths:
-            if os.path.exists(file_path):
-                assert os.stat(file_path).st_size == 0
-                os.remove(file_path)
+        remove_files(log_paths, assert_empty=True)
         # mimicking behavior of bash and zsh.
         print(f"{args.command}: command not found", file=sys.stderr)
         return 127  # seems what zsh and bash return then
@@ -1073,7 +1099,16 @@ def execute(args: Arguments) -> int:
             report.run_time_seconds = f"{report.end_time - report.start_time}"
             system_logs.write(report.dump_json())
     safe_close_files(files_to_close)
-    lgr.info(report.execution_summary_formatted)
+    if process.returncode != 0 and (
+        report.elapsed_time < args.fail_time or args.fail_time < 0
+    ):
+        lgr.info(
+            "Removing log files since command failed%s.",
+            f" in less than {args.fail_time} seconds" if args.fail_time > 0 else "",
+        )
+        remove_files(log_paths)
+    else:
+        lgr.info(report.execution_summary_formatted)
     return report.process.returncode
 
 
